@@ -1,16 +1,20 @@
 from sirius_sdk.agent.agent import Agent
 from sirius_sdk.hub import init, context, endpoints, ledger, subscribe, ping, send, send_to, \
     generate_qr_code, DID, Crypto, Microledgers, PairwiseList, AnonCreds, CoProtocolThreadedP2P, CoProtocolP2PAnon, \
-    CoProtocolP2P, AbstractP2PCoProtocol, CoProtocolThreadedTheirs, Cache
+    CoProtocolP2P, AbstractP2PCoProtocol, CoProtocolThreadedTheirs, Cache, open_communication, NonSecrets, \
+    acquire, release
 from sirius_sdk.encryption import P2PConnection
 from sirius_sdk.agent.pairwise import Pairwise, TheirEndpoint
 from sirius_sdk.agent.connections import Endpoint
 from sirius_sdk.agent import aries_rfc
+from sirius_sdk.agent.ledger import Schema, CredentialDefinition, AnonCredSchema, Ledger
+from sirius_sdk.errors import indy_exceptions, exceptions
 
 
 __all__ = [
     "Agent", "P2PConnection", "Pairwise", "TheirEndpoint", "Endpoint", "init", "context", "endpoints", "ledger",
     "subscribe", "ping", "send", "send_to", "generate_qr_code", "DID", "Crypto", "Microledgers", "PairwiseList",
     "CoProtocolThreadedP2P", "CoProtocolP2PAnon", "CoProtocolP2P", "AbstractP2PCoProtocol",
-    "aries_rfc", "CoProtocolThreadedTheirs", "AnonCreds", "Cache"
+    "aries_rfc", "CoProtocolThreadedTheirs", "AnonCreds", "Cache", "open_communication", "NonSecrets",
+    "Schema", "CredentialDefinition", "AnonCredSchema", "indy_exceptions", "exceptions", "Ledger", "acquire", "release"
 ]
