@@ -5,6 +5,7 @@ from typing import List, Optional
 from collections import UserDict
 
 from sirius_sdk.errors.exceptions import *
+from sirius_sdk.agent.aries_rfc.feature_0015_acks.messages import Ack
 from sirius_sdk.agent.aries_rfc.base import AriesProtocolMessage, RegisterMessage, AriesProblemReport, THREAD_DECORATOR
 
 
@@ -103,6 +104,11 @@ class PresentProofProblemReport(AriesProblemReport, metaclass=RegisterMessage):
     PROTOCOL = BasePresentProofMessage.PROTOCOL
 
 
+class PresentationAck(Ack):
+
+    PROTOCOL = BasePresentProofMessage.PROTOCOL
+
+
 class RequestPresentationMessage(BasePresentProofMessage):
 
     NAME = 'request-presentation'
