@@ -33,20 +33,25 @@ class Pairwise:
 
     class Their(TheirEndpoint):
 
-        def __init__(self, did: str, label: str, endpoint: str, verkey: str, routing_keys: List[str]=None):
+        def __init__(
+                self, did: str, label: str, endpoint: str, verkey: str,
+                routing_keys: List[str] = None, did_doc: dict = None
+        ):
             self.did = did
             self.label = label
+            self.did_doc = did_doc
             super().__init__(endpoint, verkey, routing_keys)
 
     class Me:
 
-        def __init__(self, did, verkey):
+        def __init__(self, did, verkey, did_doc: dict = None):
             self.did = did
             self.verkey = verkey
+            self.did_doc = did_doc
 
         def __eq__(self, other):
             if isinstance(other, Pairwise.Me):
-                return self.did == other.did and self.verkey == other.verkey
+                return self.did == other.did and self.verkey == other.verkey and self.did_doc == other.did_doc
 
     def __init__(self, me: Me, their: Their, metadata: dict=None):
         self.__me = me
@@ -132,17 +137,21 @@ class WalletPairwiseList(AbstractPairwiseList):
 
     async def create(self, pairwise: Pairwise):
         await self._api_did.store_their_did(did=pairwise.their.did, verkey=pairwise.their.verkey)
+        metadata = pairwise.metadata or {}
+        metadata.update(self._build_metadata(pairwise))
         await self._api_pairwise.create_pairwise(
             their_did=pairwise.their.did,
             my_did=pairwise.me.did,
-            metadata=pairwise.metadata,
+            metadata=metadata,
             tags=self._build_tags(pairwise)
         )
 
     async def update(self, pairwise: Pairwise):
+        metadata = pairwise.metadata or {}
+        metadata.update(self._build_metadata(pairwise))
         await self._api_pairwise.set_pairwise_metadata(
             their_did=pairwise.their.did,
-            metadata=pairwise.metadata,
+            metadata=metadata,
             tags=self._build_tags(pairwise)
         )
 
@@ -200,7 +209,8 @@ class WalletPairwiseList(AbstractPairwiseList):
         pairwise = Pairwise(
             me=Pairwise.Me(
                 did=metadata.get('me', {}).get('did', None),
-                verkey=metadata.get('me', {}).get('verkey', None)
+                verkey=metadata.get('me', {}).get('verkey', None),
+                did_doc=metadata.get('me', {}).get('did_doc', None)
             ),
             their=Pairwise.Their(
                 did=metadata.get('their', {}).get('did', None),
@@ -208,7 +218,29 @@ class WalletPairwiseList(AbstractPairwiseList):
                 label=metadata.get('their', {}).get('label', None),
                 endpoint=metadata.get('their', {}).get('endpoint', {}).get('address', None),
                 routing_keys=metadata.get('their', {}).get('endpoint', {}).get('routing_keys', None),
+                did_doc=metadata.get('their', {}).get('did_doc', None)
             ),
             metadata=metadata
         )
         return pairwise
+
+    @staticmethod
+    def _build_metadata(pairwise: Pairwise) -> dict:
+        metadata = {
+            'me': {
+                'did': pairwise.me.did,
+                'verkey': pairwise.me.verkey,
+                'did_doc': pairwise.me.did_doc
+            },
+            'their': {
+                'did': pairwise.their.did,
+                'verkey': pairwise.their.verkey,
+                'label': pairwise.their.label,
+                'endpoint': {
+                    'address': pairwise.their.endpoint,
+                    'routing_keys': pairwise.their.routing_keys
+                },
+                'did_doc': pairwise.their.did_doc
+            }
+        }
+        return metadata
