@@ -5,5 +5,6 @@ from sirius_sdk.encryption.p2p import P2PConnection
 
 __all__ = [
     "b64_to_bytes", "bytes_to_b64", "b58_to_bytes", "bytes_to_b58", "create_keypair",
-    "random_seed", "validate_seed", "pack_message", "unpack_message", "P2PConnection"
+    "random_seed", "validate_seed", "pack_message", "unpack_message", "P2PConnection", "sign_message",
+    "verify_signed_message", "did_from_verkey"
 ]
