@@ -1,6 +1,7 @@
 from sirius_sdk.messaging import *
 from sirius_sdk.agent.aries_rfc.feature_0048_trust_ping.messages import Ping, Pong
 from sirius_sdk.agent.aries_rfc.feature_0015_acks.messages import Ack, Status as AckStatus
+import sirius_sdk.agent.aries_rfc.feature_0095_basic_message.messages as msg0095
 
 
 class Test1Message(Message):
@@ -11,6 +12,24 @@ class Test2Message(Message):
     pass
 
 
+def test_type_parsing():
+    str1 = 'did:sov:BzCbsNYhMrjHiqZDTUASHg;spec/test-protocol/1.0/name'
+    typ = Type.from_str(str1)
+    assert typ.doc_uri == 'did:sov:BzCbsNYhMrjHiqZDTUASHg;spec/'
+    assert typ.protocol == 'test-protocol'
+    assert typ.name == 'name'
+    assert typ.version == '1.0'
+    assert typ.version_info.major == 1 and typ.version_info.minor == 0 and typ.version_info.patch == 0
+
+    str2 = 'https://didcomm.org/test-protocol/1.2/name'
+    typ = Type.from_str(str2)
+    assert typ.doc_uri == 'https://didcomm.org/'
+    assert typ.protocol == 'test-protocol'
+    assert typ.name == 'name'
+    assert typ.version == '1.2'
+    assert typ.version_info.major == 1 and typ.version_info.minor == 2 and typ.version_info.patch == 0
+
+
 def test_register_protocol_message_success():
     register_message_class(Test1Message, protocol='test-protocol')
     ok, msg = restore_message_instance(
@@ -128,3 +147,14 @@ def test_aries_ack():
     ack.validate()
     assert ack.status == AckStatus.PENDING
 
+
+def test_attaches_mixin():
+    msg = msg0095.Message(content="content", locale="en")
+    att = msg0095.Attach(id="id", mime_type="image/png", filename="photo.png", data="eW91ciB0ZXh0".encode())
+    msg.add_attach(att)
+
+    assert len(msg.attaches) == 1
+    assert isinstance(msg.attaches[0], msg0095.Attach)
+    assert msg.attaches[0].data == "eW91ciB0ZXh0".encode()
+
+
